# This file is generated by rake. Do not edit.

module Platform; end
module Platform::Errno
  E2BIG = 7
  EACCES = 13
  EADDRINUSE = 48
  EADDRNOTAVAIL = 49
  EAFNOSUPPORT = 47
  EAGAIN = 35
  EALREADY = 37
  EBADF = 9
  EBUSY = 16
  ECHILD = 10
  ECONNABORTED = 53
  ECONNREFUSED = 61
  ECONNRESET = 54
  EDEADLK = 11
  EDESTADDRREQ = 39
  EEXIST = 17
  EFAULT = 14
  EFBIG = 27
  EHOSTDOWN = 64
  EHOSTUNREACH = 65
  EINPROGRESS = 36
  EINTR = 4
  EINVAL = 22
  EIO = 5
  EISCONN = 56
  EISDIR = 21
  ELOOP = 62
  EMFILE = 24
  EMLINK = 31
  EMSGSIZE = 40
  ENAMETOOLONG = 63
  ENETDOWN = 50
  ENETRESET = 52
  ENETUNREACH = 51
  ENFILE = 23
  ENOBUFS = 55
  ENODEV = 19
  ENOENT = 2
  ENOEXEC = 8
  ENOMEM = 12
  ENOPROTOOPT = 42
  ENOSPC = 28
  ENOTCONN = 57
  ENOTDIR = 20
  ENOTEMPTY = 66
  ENOTSOCK = 38
  ENOTTY = 25
  ENXIO = 6
  EOPNOTSUPP = 45
  EPERM = 1
  EPIPE = 32
  EPROTONOSUPPORT = 43
  EPROTOTYPE = 41
  EROFS = 30
  ESPIPE = 29
  ESRCH = 3
  ETIMEDOUT = 60
  ETXTBSY = 26
  EWOULDBLOCK = 35
  EXDEV = 18






end

