# This file is generated by rake. Do not edit.

module Platform; end
module Platform::Etc
  class Passwd < FFI::Struct
    self.size = 72
    layout :pw_name, :string, 0,
           :pw_passwd, :string, 8,
           :pw_uid, :uint, 16,
           :pw_gid, :uint, 20,
           :pw_dir, :string, 48,
           :pw_shell, :string, 56






  end
  class Group < FFI::Struct
    self.size = 32
    layout :gr_name, :string, 0,
           :gr_gid, :uint, 16






  end
end
