# This file is generated by rake. Do not edit.

module Platform; end
module Platform::Socket
  AF_APPLETALK = 16
  # AF_ATM not available
  # AF_AX25 not available
  AF_CCITT = 10
  AF_CHAOS = 5
  AF_CNT = 21
  AF_COIP = 20
  AF_DATAKIT = 9
  AF_DECnet = 12
  AF_DLI = 13
  AF_E164 = 26
  AF_ECMA = 8
  AF_HYLINK = 15
  AF_IMPLINK = 3
  AF_INET = 2
  AF_INET6 = 24
  AF_IPX = 23
  AF_ISDN = 26
  AF_ISO = 7
  AF_LAT = 14
  AF_LINK = 18
  AF_LOCAL = 1
  AF_MAX = 35
  AF_NATM = 27
  # AF_NDRV not available
  # AF_NETBIOS not available
  # AF_NETGRAPH not available
  AF_NS = 6
  AF_OSI = 7
  # AF_PPP not available
  AF_PUP = 4
  AF_ROUTE = 17
  AF_SIP = 29
  AF_SNA = 11
  # AF_SYSTEM not available
  AF_UNIX = 1
  AF_UNSPEC = 0
  NI_DGRAM = 16
  NI_FQDN_FLAG_VALIDTTL = 256
  NI_MAXHOST = 256
  NI_MAXSERV = 32
  NI_NAMEREQD = 8
  NI_NODEADDR_FLAG_ALL = 512
  NI_NODEADDR_FLAG_ANYCAST = 16384
  NI_NODEADDR_FLAG_COMPAT = 1024
  NI_NODEADDR_FLAG_GLOBAL = 8192
  NI_NODEADDR_FLAG_LINKLOCAL = 2048
  NI_NODEADDR_FLAG_SITELOCAL = 4096
  NI_NODEADDR_FLAG_TRUNCATE = 256
  NI_NOFQDN = 4
  NI_NUMERICHOST = 1
  NI_NUMERICSERV = 2
  NI_QTYPE_DNSNAME = 2
  NI_QTYPE_FQDN = 2
  NI_QTYPE_IPV4ADDR = 4
  NI_QTYPE_NODEADDR = 3
  NI_QTYPE_NOOP = 0
  NI_QTYPE_SUPTYPES = 1
  NI_SUPTYPE_FLAG_COMPRESS = 256
  # NI_WITHSCOPEID not available
  PF_APPLETALK = 16
  # PF_ATM not available
  PF_CCITT = 10
  PF_CHAOS = 5
  PF_CNT = 21
  PF_COIP = 20
  PF_DATAKIT = 9
  PF_DECnet = 12
  PF_DLI = 13
  PF_ECMA = 8
  PF_HYLINK = 15
  PF_IMPLINK = 3
  PF_INET = 2
  PF_INET6 = 24
  PF_IPX = 23
  PF_ISDN = 26
  PF_ISO = 7
  PF_KEY = 30
  PF_LAT = 14
  PF_LINK = 18
  PF_LOCAL = 1
  PF_MAX = 35
  PF_NATM = 27
  # PF_NDRV not available
  # PF_NETBIOS not available
  # PF_NETGRAPH not available
  PF_NS = 6
  PF_OSI = 7
  PF_PIP = 25
  # PF_PPP not available
  PF_PUP = 4
  PF_ROUTE = 17
  PF_RTIP = 22
  PF_SIP = 29
  PF_SNA = 11
  # PF_SYSTEM not available
  PF_UNIX = 1
  PF_UNSPEC = 0
  PF_XTP = 19
  SOCK_DGRAM = 2
  # SOCK_MAXADDRLEN not available
  SOCK_RAW = 3
  SOCK_RDM = 4
  SOCK_SEQPACKET = 5
  SOCK_STREAM = 1
  SO_ACCEPTCONN = 2
  # SO_ACCEPTFILTER not available
  # SO_ATTACH_FILTER not available
  # SO_BINDTODEVICE not available
  SO_BROADCAST = 32
  SO_DEBUG = 1
  # SO_DETACH_FILTER not available
  SO_DONTROUTE = 16
  # SO_DONTTRUNC not available
  SO_ERROR = 4103
  SO_KEEPALIVE = 8
  # SO_LABEL not available
  SO_LINGER = 128
  # SO_NKE not available
  # SO_NOADDRERR not available
  # SO_NOSIGPIPE not available
  # SO_NO_CHECK not available
  # SO_NREAD not available
  # SO_NWRITE not available
  SO_OOBINLINE = 256
  # SO_PASSCRED not available
  SO_PEERCRED = 4130
  # SO_PEERLABEL not available
  # SO_PEERNAME not available
  # SO_PRIORITY not available
  SO_RCVBUF = 4098
  SO_RCVLOWAT = 4100
  SO_RCVTIMEO = 4102
  SO_REUSEADDR = 4
  SO_REUSEPORT = 512
  # SO_REUSESHAREUID not available
  # SO_SECURITY_AUTHENTICATION not available
  # SO_SECURITY_ENCRYPTION_NETWORK not available
  # SO_SECURITY_ENCRYPTION_TRANSPORT not available
  SO_SNDBUF = 4097
  SO_SNDLOWAT = 4099
  SO_SNDTIMEO = 4101
  SO_TIMESTAMP = 2048
  SO_TYPE = 4104
  SO_USELOOPBACK = 64
  # SO_WANTMORE not available
  # SO_WANTOOBFLAG not available
  pseudo_AF_HDRCMPLT = 31
  # pseudo_AF_KEY not available
  pseudo_AF_PIP = 25
  pseudo_AF_RTIP = 22
  pseudo_AF_XTP = 19








































end
    
