# This file is generated by rake. Do not edit.

module Platform;end
module Platform::Sysconf
  SC_2_CHAR_TERM = 20
  SC_2_C_BIND = 18
  SC_2_C_DEV = 19
  SC_2_FORT_DEV = 21
  SC_2_FORT_RUN = 22
  SC_2_LOCALEDEF = 23
  # _SC_2_PBS not available
  # _SC_2_PBS_ACCOUNTING not available
  # _SC_2_PBS_CHECKPOINT not available
  # _SC_2_PBS_LOCATE not available
  # _SC_2_PBS_MESSAGE not available
  # _SC_2_PBS_TRACK not available
  SC_2_SW_DEV = 24
  SC_2_UPE = 25
  SC_2_VERSION = 17
  # _SC_ADVISORY_INFO not available
  # _SC_AIO_LISTIO_MAX not available
  # _SC_AIO_MAX not available
  # _SC_AIO_PRIO_DELTA_MAX not available
  SC_ARG_MAX = 1
  # _SC_ASYNCHRONOUS_IO not available
  # _SC_ATEXIT_MAX not available
  # _SC_BARRIERS not available
  SC_BC_BASE_MAX = 9
  SC_BC_DIM_MAX = 10
  SC_BC_SCALE_MAX = 11
  SC_BC_STRING_MAX = 12
  SC_CHILD_MAX = 2
  SC_CLK_TCK = 3
  # _SC_CLOCK_SELECTION not available
  SC_COLL_WEIGHTS_MAX = 13
  # _SC_CPUTIME not available
  # _SC_DELAYTIMER_MAX not available
  SC_EXPR_NEST_MAX = 14
  # _SC_FILE_LOCKING not available
  SC_FSYNC = 29
  SC_GETGR_R_SIZE_MAX = 100
  SC_GETPW_R_SIZE_MAX = 101
  # _SC_HOST_NAME_MAX not available
  # _SC_IOV_MAX not available
  # _SC_IPV6 not available
  SC_JOB_CONTROL = 6
  SC_LINE_MAX = 15
  SC_LOGIN_NAME_MAX = 102
  # _SC_MAPPED_FILES not available
  # _SC_MEMLOCK not available
  # _SC_MEMLOCK_RANGE not available
  # _SC_MEMORY_PROTECTION not available
  # _SC_MESSAGE_PASSING not available
  # _SC_MONOTONIC_CLOCK not available
  # _SC_MQ_OPEN_MAX not available
  # _SC_MQ_PRIO_MAX not available
  SC_NGROUPS_MAX = 4
  SC_NPROCESSORS_CONF = 502
  SC_NPROCESSORS_ONLN = 503
  SC_OPEN_MAX = 5
  SC_PAGESIZE = 28
  SC_PAGE_SIZE = 28
  # _SC_PASS_MAX not available
  # _SC_PRIORITIZED_IO not available
  # _SC_PRIORITY_SCHEDULING not available
  # _SC_RAW_SOCKETS not available
  # _SC_READER_WRITER_LOCKS not available
  # _SC_REALTIME_SIGNALS not available
  # _SC_REGEXP not available
  SC_RE_DUP_MAX = 16
  # _SC_RTSIG_MAX not available
  SC_SAVED_IDS = 7
  # _SC_SEMAPHORES not available
  SC_SEM_NSEMS_MAX = 31
  SC_SEM_VALUE_MAX = 32
  # _SC_SHARED_MEMORY_OBJECTS not available
  # _SC_SHELL not available
  # _SC_SIGQUEUE_MAX not available
  # _SC_SPAWN not available
  # _SC_SPIN_LOCKS not available
  # _SC_SPORADIC_SERVER not available
  # _SC_SS_REPL_MAX not available
  SC_STREAM_MAX = 26
  # _SC_SYMLOOP_MAX not available
  # _SC_SYNCHRONIZED_IO not available
  # _SC_THREADS not available
  # _SC_THREAD_ATTR_STACKADDR not available
  # _SC_THREAD_ATTR_STACKSIZE not available
  # _SC_THREAD_CPUTIME not available
  # _SC_THREAD_DESTRUCTOR_ITERATIONS not available
  # _SC_THREAD_KEYS_MAX not available
  # _SC_THREAD_PRIORITY_SCHEDULING not available
  # _SC_THREAD_PRIO_INHERIT not available
  # _SC_THREAD_PRIO_PROTECT not available
  # _SC_THREAD_PROCESS_SHARED not available
  SC_THREAD_SAFE_FUNCTIONS = 103
  # _SC_THREAD_SPORADIC_SERVER not available
  # _SC_THREAD_STACK_MIN not available
  # _SC_THREAD_THREADS_MAX not available
  # _SC_TIMEOUTS not available
  # _SC_TIMERS not available
  # _SC_TIMER_MAX not available
  # _SC_TRACE not available
  # _SC_TRACE_EVENT_FILTER not available
  # _SC_TRACE_EVENT_NAME_MAX not available
  # _SC_TRACE_INHERIT not available
  # _SC_TRACE_LOG not available
  # _SC_TRACE_NAME_MAX not available
  # _SC_TRACE_SYS_MAX not available
  # _SC_TRACE_USER_EVENT_MAX not available
  # _SC_TTY_NAME_MAX not available
  # _SC_TYPED_MEMORY_OBJECTS not available
  SC_TZNAME_MAX = 27
  # _SC_V6_ILP32_OFF32 not available
  # _SC_V6_ILP32_OFFBIG not available
  # _SC_V6_LP64_OFF64 not available
  # _SC_V6_LPBIG_OFFBIG not available
  SC_VERSION = 8
  # _SC_XBS5_ILP32_OFF32 not available
  # _SC_XBS5_ILP32_OFFBIG not available
  # _SC_XBS5_LP64_OFF64 not available
  # _SC_XBS5_LPBIG_OFFBIG not available
  # _SC_XOPEN_CRYPT not available
  # _SC_XOPEN_ENH_I18N not available
  # _SC_XOPEN_LEGACY not available
  # _SC_XOPEN_REALTIME not available
  # _SC_XOPEN_REALTIME_THREADS not available
  SC_XOPEN_SHM = 30
  # _SC_XOPEN_STREAMS not available
  # _SC_XOPEN_UNIX not available
  # _SC_XOPEN_VERSION not available
  # _SC_XOPEN_XCU_VERSION not available







end
