# This file is generated by rake. Do not edit.

module Fcntl
  FD_CLOEXEC = 1
  F_DUPFD = 0
  F_GETFD = 1
  F_GETFL = 3
  F_GETLK = 7
  F_RDLCK = 1
  F_SETFD = 2
  F_SETFL = 4
  F_SETLK = 8
  F_SETLKW = 9
  F_UNLCK = 2
  F_WRLCK = 3
  O_ACCMODE = 3
  O_APPEND = 8
  O_CREAT = 512
  O_EXCL = 2048
  O_NDELAY = 4
  O_NOCTTY = 32768
  O_NONBLOCK = 4
  O_RDONLY = 0
  O_RDWR = 2
  O_TRUNC = 1024
  O_WRONLY = 1





end

